@@ -15,8 +15,8 @@
 // commercial license or contractual agreement.
 
 
-#include <Adaptor2d_HCurve2d.hxx>
-#include <Adaptor3d_HSurface.hxx>
+#include <Adaptor2d_Curve2d.hxx>
+#include <Adaptor3d_Surface.hxx>
 #include <Adaptor3d_HVertex.hxx>
 #include <Adaptor3d_TopolTool.hxx>
 #include <Blend_FuncInv.hxx>
@@ -96,28 +96,28 @@ static void tracederiv(Blend_SurfRstFunction& Func,
   Standard_Real param2 = BP2.Parameter();
   Standard_Real scal = 1./(param1-param2);
 
-  cout<<endl;
-  cout<<"control derivatives at point : "<<param1<<endl;
+  std::cout<<std::endl;
+  std::cout<<"control derivatives at point : "<<param1<<std::endl;
 
   for(i = 1; i <= hp; i++){
-    cout<<endl;
-    cout<<"point : "<<i<<endl;
-    cout<<"dx calculated : "<<TDP1(i).X()<<endl;
-    cout<<"dx estimated  : "<<scal*(TP1(i).X()-TP2(i).X())<<endl;
-    cout<<"dy calculated : "<<TDP1(i).Y()<<endl;
-    cout<<"dy estimated  : "<<scal*(TP1(i).Y()-TP2(i).Y())<<endl;
-    cout<<"dz calculated : "<<TDP1(i).Z()<<endl;
-    cout<<"dz estimated  : "<<scal*(TP1(i).Z()-TP2(i).Z())<<endl;
-    cout<<"dw calculated : "<<TDW1(i)<<endl;
-    cout<<"dw estimated  : "<<scal*(TW1(i)-TW2(i))<<endl;
+    std::cout<<std::endl;
+    std::cout<<"point : "<<i<<std::endl;
+    std::cout<<"dx calculated : "<<TDP1(i).X()<<std::endl;
+    std::cout<<"dx estimated  : "<<scal*(TP1(i).X()-TP2(i).X())<<std::endl;
+    std::cout<<"dy calculated : "<<TDP1(i).Y()<<std::endl;
+    std::cout<<"dy estimated  : "<<scal*(TP1(i).Y()-TP2(i).Y())<<std::endl;
+    std::cout<<"dz calculated : "<<TDP1(i).Z()<<std::endl;
+    std::cout<<"dz estimated  : "<<scal*(TP1(i).Z()-TP2(i).Z())<<std::endl;
+    std::cout<<"dw calculated : "<<TDW1(i)<<std::endl;
+    std::cout<<"dw estimated  : "<<scal*(TW1(i)-TW2(i))<<std::endl;
   }
   for(i = 1; i <= hp2d; i++){
-    cout<<endl;
-    cout<<"point 2d : "<<i<<endl;
-    cout<<"dx calculated : "<<TDP2d1(i).X()<<endl;
-    cout<<"dx estimated  : "<<scal*(TP2d1(i).X()-TP2d2(i).X())<<endl;
-    cout<<"dy calculated : "<<TDP2d1(i).Y()<<endl;
-    cout<<"dy estimated  : "<<scal*(TP2d1(i).Y()-TP2d2(i).Y())<<endl;
+    std::cout<<std::endl;
+    std::cout<<"point 2d : "<<i<<std::endl;
+    std::cout<<"dx calculated : "<<TDP2d1(i).X()<<std::endl;
+    std::cout<<"dx estimated  : "<<scal*(TP2d1(i).X()-TP2d2(i).X())<<std::endl;
+    std::cout<<"dy calculated : "<<TDP2d1(i).Y()<<std::endl;
+    std::cout<<"dy estimated  : "<<scal*(TP2d1(i).Y()-TP2d2(i).Y())<<std::endl;
   }
 }
 #endif
@@ -207,13 +207,18 @@ Standard_Integer BRepBlend_SurfRstLineBuilder::
 //=======================================================================
 
 BRepBlend_SurfRstLineBuilder::BRepBlend_SurfRstLineBuilder
-(const Handle(Adaptor3d_HSurface)&  Surf1,
+(const Handle(Adaptor3d_Surface)&  Surf1,
  const Handle(Adaptor3d_TopolTool)& Domain1,
- const Handle(Adaptor3d_HSurface)&  Surf2,
- const Handle(Adaptor2d_HCurve2d)&  Rst,
+ const Handle(Adaptor3d_Surface)&  Surf2,
+ const Handle(Adaptor2d_Curve2d)&  Rst,
  const Handle(Adaptor3d_TopolTool)& Domain2):
- sol(1,3),surf1(Surf1), domain1(Domain1),
- surf2(Surf2), rst(Rst), domain2(Domain2)
+ done(Standard_False), sol(1, 3), surf1(Surf1),
+ domain1(Domain1), surf2(Surf2), rst(Rst),
+ domain2(Domain2), tolesp(0.0), tolgui(0.0),
+ pasmax(0.0), fleche(0.0), param(0.0),
+ rebrou(Standard_False), iscomplete(Standard_False),
+ comptra(Standard_False), sens(0.0),
+ decrochdeb(Standard_False), decrochfin(Standard_False)
 {
 }
 
@@ -357,7 +362,7 @@ Standard_Boolean BRepBlend_SurfRstLineBuilder::PerformFirstSection
   math_Vector solinvp(1,3),solinvrst(1,4),solinvs(1,3);
   Handle(Adaptor3d_HVertex) Vtxp,Vtxrst,Vtxs,Vtxc;
   Standard_Boolean IsVtxp = 0,IsVtxrst = 0,IsVtxs = 0;
-  Handle(Adaptor2d_HCurve2d) Arc;
+  Handle(Adaptor2d_Curve2d) Arc;
   wp = wrst = ws = Pmax;
   param = Pdep;
   Func.Set(param);
@@ -531,7 +536,7 @@ void BRepBlend_SurfRstLineBuilder::InternalPerform(Blend_SurfRstFunction&  Func,
   Handle(Adaptor3d_HVertex) Vtxp,Vtxrst,Vtxs,Vtxc;
   Standard_Boolean IsVtxp = 0,IsVtxrst = 0,IsVtxs = 0;
   BRepBlend_Extremity Extrst,Exts;
-  Handle(Adaptor2d_HCurve2d) Arc;
+  Handle(Adaptor2d_Curve2d) Arc;
 
   //IntSurf_Transition Tline,Tarc;
 
@@ -717,7 +722,7 @@ void BRepBlend_SurfRstLineBuilder::InternalPerform(Blend_SurfRstFunction&  Func,
       else{
 	// Failed reframing. Leave with PointsConfondus
 #ifdef OCCT_DEBUG
-	cout<<"SurfRstLineBuilder : failed reframing"<<endl;
+	std::cout<<"SurfRstLineBuilder : failed reframing"<<std::endl;
 #endif
 	State = Blend_SamePoints;
       }
@@ -772,7 +777,7 @@ void BRepBlend_SurfRstLineBuilder::InternalPerform(Blend_SurfRstFunction&  Func,
 	  if (line->NbPoints()>=2) {
 	    // Indicate that one stops during the processing
 #ifdef OCCT_DEBUG
-	    cout<<"SurfRstLineBuilder : No advancement in the processing"<<endl;
+	    std::cout<<"SurfRstLineBuilder : No advancement in the processing"<<std::endl;
 #endif
 	  }
 	}
@@ -878,7 +883,7 @@ void BRepBlend_SurfRstLineBuilder::InternalPerform(Blend_SurfRstFunction&  Func,
       {
 	// Stop
 #ifdef OCCT_DEBUG
-	cout << "SurfRstLineBuilder Points mixed in the processing" << endl;
+	std::cout << "SurfRstLineBuilder Points mixed in the processing" << std::endl;
 #endif
 	previousP.ParametersOnS(U,V);
 	Exts.SetValue(previousP.PointOnS(),U,V,
@@ -912,7 +917,7 @@ void BRepBlend_SurfRstLineBuilder::InternalPerform(Blend_SurfRstFunction&  Func,
 
 Standard_Boolean BRepBlend_SurfRstLineBuilder::Recadre(Blend_SurfCurvFuncInv&    FinvC,
 						       math_Vector&              Solinv,
-						       Handle(Adaptor2d_HCurve2d)& Arc,
+						       Handle(Adaptor2d_Curve2d)& Arc,
 						       Standard_Boolean&         IsVtx,
 						       Handle(Adaptor3d_HVertex)&  Vtx) 
 {
@@ -951,7 +956,7 @@ Standard_Boolean BRepBlend_SurfRstLineBuilder::Recadre(Blend_SurfCurvFuncInv&
 
   if (!rsnld.IsDone()) {
 #ifdef OCCT_DEBUG
-    cout << "SurfRstLineBuilder : RSNLD not done "<< endl << endl;
+    std::cout << "SurfRstLineBuilder : RSNLD not done "<< std::endl << std::endl;
 #endif
   }
   else {
@@ -990,7 +995,7 @@ Standard_Boolean BRepBlend_SurfRstLineBuilder::Recadre(Blend_SurfCurvFuncInv&
 
     if (!aRsnld.IsDone()) {
 #ifdef OCCT_DEBUG
-      cout << "SurfRstLineBuilder : RSNLD not done "<< endl << endl;
+      std::cout << "SurfRstLineBuilder : RSNLD not done "<< std::endl << std::endl;
 #endif
     }
     else {
@@ -1051,7 +1056,7 @@ Standard_Boolean BRepBlend_SurfRstLineBuilder::Recadre(Blend_SurfRstFunction&
   rsnld.Perform(Finv,Solinv,infb,supb);
   if (!rsnld.IsDone()) {
 #ifdef OCCT_DEBUG
-    cout << "SurfRstLineBuilder :RSNLD not done "<< endl;
+    std::cout << "SurfRstLineBuilder :RSNLD not done "<< std::endl;
 #endif
     return Standard_False;
   }
@@ -1135,7 +1140,7 @@ Standard_Boolean BRepBlend_SurfRstLineBuilder::Recadre(Blend_SurfPointFuncInv&
   rsnld.Perform(FinvP,Solinv,infb,supb);
   if (!rsnld.IsDone()) {
 #ifdef OCCT_DEBUG
-    cout << "SurfRstLineBuilder :RSNLD not done "<< endl;
+    std::cout << "SurfRstLineBuilder :RSNLD not done "<< std::endl;
 #endif
     return Standard_False;
   }
@@ -1175,7 +1180,7 @@ Standard_Boolean BRepBlend_SurfRstLineBuilder::Recadre(Blend_SurfPointFuncInv&
 //=======================================================================
 
 void BRepBlend_SurfRstLineBuilder::Transition(const Standard_Boolean          OnFirst,
-					      const Handle(Adaptor2d_HCurve2d)& Arc,
+					      const Handle(Adaptor2d_Curve2d)& Arc,
 					      const Standard_Real             Param,
 					      IntSurf_Transition&             TLine,
 					      IntSurf_Transition&             TArc) 
@@ -1225,7 +1230,7 @@ void BRepBlend_SurfRstLineBuilder::Transition(const Standard_Boolean          On
 
 void BRepBlend_SurfRstLineBuilder::MakeExtremity(BRepBlend_Extremity&            Extrem,
 						 const Standard_Boolean          OnFirst,
-						 const Handle(Adaptor2d_HCurve2d)& Arc,
+						 const Handle(Adaptor2d_Curve2d)& Arc,
 						 const Standard_Real             Param,
 						 const Standard_Boolean          IsVtx,
 						 const Handle(Adaptor3d_HVertex)&  Vtx) 
@@ -1258,7 +1263,7 @@ void BRepBlend_SurfRstLineBuilder::MakeExtremity(BRepBlend_Extremity&
   else {
     Extrem.SetVertex(Vtx);
     while (Iter->More()) {
-      Handle(Adaptor2d_HCurve2d) arc = Iter->Value();
+      Handle(Adaptor2d_Curve2d) arc = Iter->Value();
       if (arc != Arc) {
 	Iter->Initialize(arc);
 	Iter->InitVertexIterator();
