@@ -21,12 +21,13 @@
 #include <BinMDF_ADriver.hxx>
 #include <BinMDF_ADriverTable.hxx>
 #include <BinObjMgt_Persistent.hxx>
+#include <BinObjMgt_Position.hxx>
 #include <CDM_Application.hxx>
 #include <CDM_Document.hxx>
 #include <Message_Messenger.hxx>
 #include <FSD_BinaryFile.hxx>
 #include <FSD_FileHeader.hxx>
-#include <OSD_OpenFile.hxx>
+#include <OSD_FileSystem.hxx>
 #include <PCDM_ReadWriter.hxx>
 #include <Standard_ErrorHandler.hxx>
 #include <Standard_Type.hxx>
@@ -42,17 +43,19 @@
 #include <TDF_Label.hxx>
 #include <TDF_Tool.hxx>
 #include <TDocStd_Document.hxx>
+#include <Message_ProgressScope.hxx>
 
 IMPLEMENT_STANDARD_RTTIEXT(BinLDrivers_DocumentStorageDriver,PCDM_StorageDriver)
 
 #define SHAPESECTION_POS (Standard_CString)"SHAPE_SECTION_POS:"
+#define ENDSECTION_POS (Standard_CString)":"
 
 //=======================================================================
 //function : BinLDrivers_DocumentStorageDriver
 //purpose  : Constructor
 //=======================================================================
 
-BinLDrivers_DocumentStorageDriver::BinLDrivers_DocumentStorageDriver ()
+BinLDrivers_DocumentStorageDriver::BinLDrivers_DocumentStorageDriver()
 {
 }
 
@@ -63,19 +66,20 @@ BinLDrivers_DocumentStorageDriver::BinLDrivers_DocumentStorageDriver ()
 
 void BinLDrivers_DocumentStorageDriver::Write
                           (const Handle(CDM_Document)&       theDocument,
-                           const TCollection_ExtendedString& theFileName)
+                           const TCollection_ExtendedString& theFileName,
+                           const Message_ProgressRange&      theRange)
 {
   SetIsError(Standard_False);
   SetStoreStatus(PCDM_SS_OK);
 
   myFileName = theFileName;
 
-  std::ofstream aFileStream;
-  OSD_OpenStream (aFileStream, theFileName, std::ios::out | std::ios::binary);
+  const Handle(OSD_FileSystem)& aFileSystem = OSD_FileSystem::DefaultFileSystem();
+  opencascade::std::shared_ptr<std::ostream> aFileStream = aFileSystem->OpenOStream (theFileName, std::ios::out | std::ios::binary);
 
-  if (aFileStream.is_open() && aFileStream.good())
+  if (aFileStream.get() != NULL && aFileStream->good())
   {
-    Write (theDocument, aFileStream);
+    Write (theDocument, *aFileStream, theRange);
   }
   else
   {
@@ -89,13 +93,15 @@ void BinLDrivers_DocumentStorageDriver::Write
 //purpose  :
 //=======================================================================
 
-void BinLDrivers_DocumentStorageDriver::Write (const Handle(CDM_Document)& theDoc, Standard_OStream& theOStream)
+void BinLDrivers_DocumentStorageDriver::Write (const Handle(CDM_Document)&  theDoc, 
+                                               Standard_OStream&            theOStream,
+                                               const Message_ProgressRange& theRange)
 {
   myMsgDriver = theDoc->Application()->MessageDriver();
   myMapUnsupported.Clear();
+  mySizesToWrite.Clear();
 
-  Handle(TDocStd_Document) aDoc =
-    Handle(TDocStd_Document)::DownCast(theDoc);
+  Handle(TDocStd_Document) aDoc = Handle(TDocStd_Document)::DownCast (theDoc);
   if (aDoc.IsNull()) {
     SetIsError(Standard_True);
     SetStoreStatus(PCDM_SS_Doc_IsNull);
@@ -119,37 +125,74 @@ void BinLDrivers_DocumentStorageDriver::Write (const Handle(CDM_Document)& theDo
         return;
     }
 
-
-
 //  2. Write the Table of Contents of Sections
+    const TDocStd_FormatVersion aDocVer = aDoc->StorageFormatVersion();
     BinLDrivers_VectorOfDocumentSection::Iterator anIterS (mySections);
     for (; anIterS.More(); anIterS.Next())
-      anIterS.ChangeValue().WriteTOC (theOStream);
+      anIterS.ChangeValue().WriteTOC (theOStream, aDocVer);
 
-    // Shapes Section is the last one, it indicates the end of the table.
-    BinLDrivers_DocumentSection aShapesSection (SHAPESECTION_POS,
-                                                Standard_False);
-    aShapesSection.WriteTOC (theOStream);
+    EnableQuickPartWriting (myMsgDriver, IsQuickPart (aDocVer));
+    BinLDrivers_DocumentSection* aShapesSection = 0;
+    Standard_Boolean aQuickPart = IsQuickPart (aDocVer);
+    if (!aQuickPart)
+    {
+      // Shapes Section is the last one, it indicates the end of the table.
+      aShapesSection = new BinLDrivers_DocumentSection (SHAPESECTION_POS, Standard_False);
+      aShapesSection->WriteTOC (theOStream, aDocVer);
+    }
+    else
+    {
+      // End Section is the last one, it indicates the end of the table.
+      BinLDrivers_DocumentSection anEndSection (ENDSECTION_POS, Standard_False);
+      anEndSection.WriteTOC (theOStream, aDocVer);
+    }
 
 //  3. Write document contents
     // (Storage data to the stream)
     myRelocTable.Clear();
     myPAtt.Init();
+    if (aQuickPart)
+      myPAtt.SetOStream (theOStream); // for writing shapes data into the stream directly
+
+    Message_ProgressScope aPS(theRange, "Writing document", 3);
 
 //  Write Doc structure
-    WriteSubTree (aData->Root(), theOStream); // Doc is written
+    WriteSubTree (aData->Root(), theOStream, aQuickPart, aPS.Next()); // Doc is written
+    if (!aPS.More())
+    {
+      SetIsError(Standard_True);
+      SetStoreStatus(PCDM_SS_UserBreak);
+      return;
+    }
 
 //  4. Write Shapes section
-    WriteShapeSection (aShapesSection, theOStream);
+    if (!aQuickPart)
+    {
+      WriteShapeSection (*aShapesSection, theOStream, aDocVer, aPS.Next());
+      delete aShapesSection;
+    }
+    else
+      Clear();
+
+    if (!aPS.More())
+    {
+      SetIsError (Standard_True);
+      SetStoreStatus (PCDM_SS_UserBreak);
+      return;
+    }
 
     // Write application-defined sections
     for (anIterS.Init (mySections); anIterS.More(); anIterS.Next()) {
       BinLDrivers_DocumentSection& aSection = anIterS.ChangeValue();
       const Standard_Size aSectionOffset = (Standard_Size) theOStream.tellp();
       WriteSection (aSection.Name(), aDoc, theOStream);
-      aSection.Write (theOStream, aSectionOffset);
+      aSection.Write (theOStream, aSectionOffset, aDocVer);
     }
 
+//  5. Write sizes along the file where it is needed for quick part mode
+    if (aQuickPart)
+      WriteSizes (theOStream);
+
     // End of processing: close structures and check the status
     myPAtt.Destroy();   // free buffer
     myEmptyLabels.Clear();
@@ -164,7 +207,13 @@ void BinLDrivers_DocumentStorageDriver::Write (const Handle(CDM_Document)& theDo
       SetStoreStatus(PCDM_SS_No_Obj);
     }
     myRelocTable.Clear();
-
+    if (!aPS.More())
+    {
+      SetIsError(Standard_True);
+      SetStoreStatus(PCDM_SS_UserBreak);
+      return;
+    }
+    aPS.Next();
     if (!theOStream) {
       // A problem with the stream
 #ifdef OCCT_DEBUG
@@ -187,8 +236,8 @@ void BinLDrivers_DocumentStorageDriver::UnsupportedAttrMsg
                         (const Handle(Standard_Type)& theType)
 {
 #ifdef OCCT_DEBUG
-  static TCollection_ExtendedString aMsg
-    ("BinDrivers_DocumentStorageDriver: warning: attribute driver for type ");
+  TCollection_ExtendedString aMsg
+    ("BinLDrivers_DocumentStorageDriver: warning: attribute driver for type ");
 #endif
   if (!myMapUnsupported.Contains(theType)) {
     myMapUnsupported.Add(theType);
@@ -204,30 +253,39 @@ void BinLDrivers_DocumentStorageDriver::UnsupportedAttrMsg
 //=======================================================================
 
 void BinLDrivers_DocumentStorageDriver::WriteSubTree
-                        (const TDF_Label&          theLabel,
-                         Standard_OStream&         theOS)
+                        (const TDF_Label&             theLabel,
+                         Standard_OStream&            theOS,
+                         const Standard_Boolean&      theQuickPart,
+                         const Message_ProgressRange& theRange)
 {
   // Skip empty labels
   if (!myEmptyLabels.IsEmpty() && myEmptyLabels.First() == theLabel) {
     myEmptyLabels.RemoveFirst();
     return;
   }
-
+  Message_ProgressScope aPS(theRange, "Writing sub tree", 2, true);
   // Write label header: tag
   Standard_Integer aTag = theLabel.Tag();
-#if DO_INVERSE
+#ifdef DO_INVERSE
   aTag = InverseInt (aTag);
 #endif
   theOS.write ((char*)&aTag, sizeof(Standard_Integer));
 
+  Handle(BinObjMgt_Position) aPosition;
+  if (theQuickPart)
+  {
+    aPosition = mySizesToWrite.Append (new BinObjMgt_Position (theOS));
+    aPosition->WriteSize (theOS, Standard_True);
+  }
+
   // Write attributes
   TDF_AttributeIterator itAtt (theLabel);
-  for ( ; itAtt.More() && theOS; itAtt.Next()) {
-    const Handle(TDF_Attribute)& tAtt = itAtt.Value();
+  for ( ; itAtt.More() && theOS && aPS.More(); itAtt.Next()) {
+    const Handle(TDF_Attribute) tAtt = itAtt.Value();
     const Handle(Standard_Type)& aType = tAtt->DynamicType();
     // Get type ID and driver
     Handle(BinMDF_ADriver) aDriver;
-    const Standard_Integer aTypeId = myDrivers->GetDriver (aType,aDriver);
+    const Standard_Integer aTypeId = myDrivers->GetDriver (aType, aDriver);
     if (aTypeId > 0) {
       // Add source to relocation table
       const Standard_Integer anId = myRelocTable.Add (tAtt);
@@ -236,6 +294,12 @@ void BinLDrivers_DocumentStorageDriver::WriteSubTree
       myPAtt.SetTypeId (aTypeId);
       myPAtt.SetId (anId);
       aDriver->Paste (tAtt, myPAtt, myRelocTable);
+      if (!myPAtt.StreamStart().IsNull())
+      {
+        Handle(BinObjMgt_Position) anAttrPosition = myPAtt.StreamStart();
+        anAttrPosition->StoreSize (theOS);
+        mySizesToWrite.Append (anAttrPosition);
+      }
 
       // Write data to the stream -->!!!
       theOS << myPAtt;
@@ -249,10 +313,15 @@ void BinLDrivers_DocumentStorageDriver::WriteSubTree
     // Problem with the stream
     return;
   }
-
+  if (!aPS.More())
+  {
+    SetIsError(Standard_True);
+    SetStoreStatus(PCDM_SS_UserBreak);
+    return;
+  }
   // Write the end attributes list marker
   BinLDrivers_Marker anEndAttr = BinLDrivers_ENDATTRLIST;
-#if DO_INVERSE
+#ifdef DO_INVERSE
   anEndAttr = (BinLDrivers_Marker) InverseInt (anEndAttr);
 #endif
   theOS.write ((char*)&anEndAttr, sizeof(anEndAttr));
@@ -262,16 +331,22 @@ void BinLDrivers_DocumentStorageDriver::WriteSubTree
   for ( ; itChld.More(); itChld.Next())
   {
     const TDF_Label& aChildLab = itChld.Value();
-    WriteSubTree (aChildLab, theOS);
+    if (!aPS.More())
+    {
+      SetIsError(Standard_True);
+      SetStoreStatus(PCDM_SS_UserBreak);
+      return;
+    }
+    WriteSubTree (aChildLab, theOS, theQuickPart, aPS.Next());
   }
-
   // Write the end label marker
   BinLDrivers_Marker anEndLabel = BinLDrivers_ENDLABEL;
-#if DO_INVERSE
-  anEndLabel = (BinLDrivers_Marker) InverseInt (anEndLabel);
+#ifdef DO_INVERSE
+  anEndLabel = (BinLDrivers_Marker)InverseInt (anEndLabel);
 #endif
-  theOS.write ((char*)&anEndLabel, sizeof(anEndLabel));
-
+  theOS.write ((char*)&anEndLabel, sizeof (anEndLabel));
+  if (theQuickPart)
+    aPosition->StoreSize (theOS);
 }
 
 //=======================================================================
@@ -352,7 +427,7 @@ void BinLDrivers_DocumentStorageDriver::FirstPass
 
 //=======================================================================
 //function : WriteInfoSection
-//purpose  : Write info secton using FSD_BinaryFile driver
+//purpose  : Write info section using FSD_BinaryFile driver
 //=======================================================================
 
 #define START_TYPES "START_TYPES"
@@ -433,9 +508,11 @@ void BinLDrivers_DocumentStorageDriver::WriteInfoSection
   theData->SetApplicationVersion(theDoc->Application()->Version());
   theData->SetApplicationName(theDoc->Application()->Name());
 
+  Handle(TDocStd_Document) aDoc = Handle(TDocStd_Document)::DownCast(theDoc);
+  const Standard_Integer aDocVer = aDoc->StorageFormatVersion();
   aHeader.einfo += FSD_BinaryFile::WriteInfo (theOStream,
                                               aObjNb,
-                                              BinLDrivers::StorageVersion(),
+                                              aDocVer,
                                               Storage_Schema::ICreationDate(),
                                               "", // schema name
                                               aShemaVer,
@@ -464,7 +541,7 @@ void BinLDrivers_DocumentStorageDriver::WriteInfoSection
   // write info section
   FSD_BinaryFile::WriteInfo (theOStream,
                              aObjNb,
-                             BinLDrivers::StorageVersion(),
+                             aDocVer,
                              Storage_Schema::ICreationDate(),
                              "", // schema name
                              aShemaVer,
@@ -509,8 +586,41 @@ void BinLDrivers_DocumentStorageDriver::WriteSection
 //=======================================================================
 void BinLDrivers_DocumentStorageDriver::WriteShapeSection
                                 (BinLDrivers_DocumentSection&   theSection,
-                                 Standard_OStream&              theOS)
+                                 Standard_OStream&              theOS,
+                                 const TDocStd_FormatVersion    theDocVer,
+                                 const Message_ProgressRange& /*theRange*/)
 {
   const Standard_Size aShapesSectionOffset = (Standard_Size) theOS.tellp();
-  theSection.Write (theOS, aShapesSectionOffset);
+  theSection.Write (theOS, aShapesSectionOffset, theDocVer);
+}
+
+//=======================================================================
+//function : IsQuickPart
+//purpose  : Return true if document should be stored in quick mode for partial reading
+//=======================================================================
+Standard_Boolean BinLDrivers_DocumentStorageDriver::IsQuickPart (const Standard_Integer theVersion) const
+{
+  return theVersion >= TDocStd_FormatVersion_VERSION_12;
+}
+
+//=======================================================================
+//function : Clear
+//purpose  : 
+//=======================================================================
+void BinLDrivers_DocumentStorageDriver::Clear()
+{
+  // empty; should be redefined in subclasses
+}
+
+
+//=======================================================================
+//function : WriteSizes
+//purpose  : 
+//=======================================================================
+void BinLDrivers_DocumentStorageDriver::WriteSizes (Standard_OStream& theOS)
+{
+  NCollection_List<Handle(BinObjMgt_Position)>::Iterator anIter (mySizesToWrite);
+  for (; anIter.More() && theOS; anIter.Next())
+    anIter.Value()->WriteSize (theOS);
+  mySizesToWrite.Clear();
 }
