@@ -34,8 +34,8 @@
 #include <Precision.hxx>
 #include <ElCLib.hxx>
 #include <Adaptor3d_Curve.hxx>
-#include <Adaptor3d_HCurve.hxx>
-#include <GeomAdaptor_HCurve.hxx>
+#include <Adaptor3d_Curve.hxx>
+#include <GeomAdaptor_Curve.hxx>
 #include <Geom_Line.hxx>
 #include <GeomConvert.hxx>
 #include <BSplCLib.hxx>
@@ -44,6 +44,9 @@
 #include <Geom_Parabola.hxx>
 #include <Geom_Hyperbola.hxx>
 #include <Geom_Ellipse.hxx>
+#include <GeomLib_Tool.hxx>
+#include <math_Jacobi.hxx>
+#include <math_Matrix.hxx>
 
 
 
@@ -53,7 +56,7 @@
 //=======================================================================
 
 static gp_Pnt OnPlane_Value(const Standard_Real U,
-			    const Handle(Adaptor3d_HCurve)& aCurvePtr,
+			    const Handle(Adaptor3d_Curve)& aCurvePtr,
 			    const gp_Ax3& Pl,
 			    const gp_Dir& D)
 {
@@ -79,7 +82,7 @@ static gp_Pnt OnPlane_Value(const Standard_Real U,
 
 static gp_Vec OnPlane_DN(const Standard_Real U,
 			 const Standard_Integer DerivativeRequest,
-			 const Handle(Adaptor3d_HCurve)& aCurvePtr,
+			 const Handle(Adaptor3d_Curve)& aCurvePtr,
 			 const gp_Ax3& Pl,
 			 const gp_Dir& D)
 {
@@ -107,7 +110,7 @@ static gp_Vec OnPlane_DN(const Standard_Real U,
 static Standard_Boolean OnPlane_D1(const Standard_Real U,
 				         gp_Pnt& P,
 				         gp_Vec& V,
-				   const Handle(Adaptor3d_HCurve)& aCurvePtr,
+				   const Handle(Adaptor3d_Curve)& aCurvePtr,
 				   const gp_Ax3& Pl,
 				   const gp_Dir& D)
 {
@@ -149,7 +152,7 @@ static Standard_Boolean OnPlane_D2(const Standard_Real U,
 				         gp_Pnt& P,
 				         gp_Vec& V1,
 				         gp_Vec& V2,
-				   const Handle(Adaptor3d_HCurve) & aCurvePtr,
+				   const Handle(Adaptor3d_Curve) & aCurvePtr,
 				   const gp_Ax3& Pl,
 				   const gp_Dir& D)
 {
@@ -197,7 +200,7 @@ static Standard_Boolean OnPlane_D3(const Standard_Real U,
 				         gp_Vec& V1,
 				         gp_Vec& V2,
 				         gp_Vec& V3,
-				   const Handle(Adaptor3d_HCurve)& aCurvePtr,
+				   const Handle(Adaptor3d_Curve)& aCurvePtr,
 				   const gp_Ax3& Pl,
 				   const gp_Dir& D)
 {
@@ -248,13 +251,13 @@ static Standard_Boolean OnPlane_D3(const Standard_Real U,
 class ProjLib_OnPlane : public AppCont_Function
 
 {
-  Handle(Adaptor3d_HCurve) myCurve;
+  Handle(Adaptor3d_Curve) myCurve;
   gp_Ax3 myPlane;
   gp_Dir myDirection;
 
 public :
 
-  ProjLib_OnPlane(const Handle(Adaptor3d_HCurve)& C, 
+  ProjLib_OnPlane(const Handle(Adaptor3d_Curve)& C, 
                   const gp_Ax3& Pl, 
                   const gp_Dir& D) 
 : myCurve(C),
@@ -306,7 +309,7 @@ public :
 //purpose  : 
 //=======================================================================
 
-static void  PerformApprox (const Handle(Adaptor3d_HCurve)& C,
+static void  PerformApprox (const Handle(Adaptor3d_Curve)& C,
 			    const gp_Ax3& Pl,
 			    const gp_Dir& D,
 			    Handle(Geom_BSplineCurve) &BSplineCurvePtr) 
@@ -317,8 +320,14 @@ static void  PerformApprox (const Handle(Adaptor3d_HCurve)& C,
   Standard_Integer Deg1, Deg2;
   Deg1 = 8; Deg2 = 8;
   
-  Approx_FitAndDivide Fit(F,Deg1,Deg2,Precision::Approximation(),
+  Approx_FitAndDivide Fit(Deg1,Deg2,Precision::Approximation(),
 			  Precision::PApproximation(),Standard_True);
+  Fit.SetMaxSegments(100);
+  Fit.Perform(F);
+  if (!Fit.IsAllApproximated())
+  {
+    return;
+  }
   Standard_Integer i;
   Standard_Integer NbCurves = Fit.NbMultiCurves();
   Standard_Integer MaxDeg = 0;
@@ -347,9 +356,8 @@ static void  PerformApprox (const Handle(Adaptor3d_HCurve)& C,
     MC.Curve(1, LocalPoles);
     
     //Augmentation eventuelle du degre
-    Standard_Integer Inc = MaxDeg - MC.Degree();
-    if ( Inc > 0) {
-      BSplCLib::IncreaseDegree(Inc, Poles, BSplCLib::NoWeights(),
+    if (MaxDeg > MC.Degree() ) {
+      BSplCLib::IncreaseDegree(MaxDeg, LocalPoles, BSplCLib::NoWeights(),
 			       TempPoles, BSplCLib::NoWeights());
       //mise a jour des poles de la PCurve
       for (Standard_Integer j = 1 ; j <= MaxDeg + 1; j++) {
@@ -438,6 +446,35 @@ myIsApprox  (Standard_False)
 //  }
 }
 
+//=======================================================================
+//function : ShallowCopy
+//purpose  : 
+//=======================================================================
+
+Handle(Adaptor3d_Curve) ProjLib_ProjectOnPlane::ShallowCopy() const
+{
+  Handle(ProjLib_ProjectOnPlane) aCopy = new ProjLib_ProjectOnPlane();
+
+  if (!myCurve.IsNull())
+  {
+    aCopy->myCurve = myCurve->ShallowCopy();
+  }
+  aCopy->myPlane = myPlane;
+  aCopy->myDirection = myDirection;
+  aCopy->myKeepParam = myKeepParam;
+  aCopy->myFirstPar = myFirstPar;
+  aCopy->myLastPar = myLastPar;
+  aCopy->myTolerance = myTolerance;
+  aCopy->myType = myType;
+  if (!myResult.IsNull())
+  {
+    aCopy->myResult = Handle(GeomAdaptor_Curve)::DownCast(myResult->ShallowCopy());
+  }
+  aCopy->myIsApprox = myIsApprox;
+
+  return aCopy;
+}
+
 //=======================================================================
 //function : Project
 //purpose  : Returns the projection of a point <Point> on a plane 
@@ -483,7 +520,7 @@ static gp_Vec ProjectVec(const gp_Ax3& ThePlane,
 //purpose  : 
 //=======================================================================
 
-void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
+void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_Curve)&    C,
 				  const Standard_Real Tolerance,
 				  const Standard_Boolean KeepParametrization) 
 				  
@@ -494,7 +531,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
   myTolerance  = Tolerance ;
 
   Handle(Geom_BSplineCurve)  ApproxCurve;
-  Handle(GeomAdaptor_HCurve) aGAHCurve;
+  Handle(GeomAdaptor_Curve) aGAHCurve;
 
   Handle(Geom_Line)      GeomLinePtr;
   Handle(Geom_Circle)    GeomCirclePtr ;
@@ -511,10 +548,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
   gp_Ax2 Axis;
   Standard_Real R1 =0., R2 =0.;
 
-  if ( Type != GeomAbs_Line)  // on garde le parametrage
-    myKeepParam = Standard_True;
-  else                        // on prend le choix utilisateur.
-    myKeepParam = KeepParametrization;
+  myKeepParam = KeepParametrization;
 
   switch ( Type) {
   case GeomAbs_Line: 
@@ -539,7 +573,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
 
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
 	GeomAdaptor_Curve aGACurve(BSP);
-	myResult = new GeomAdaptor_HCurve(aGACurve);
+	myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
       }
       else if ( Abs( Xc.Magnitude() - 1.) < Precision::Confusion()) {
@@ -554,7 +588,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
 	GeomAdaptor_Curve aGACurve(GeomLinePtr,
 				   myCurve->FirstParameter(),
 				   myCurve->LastParameter() );
-	myResult = new GeomAdaptor_HCurve(aGACurve);
+	myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
       }
       else {
@@ -578,7 +612,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
 	  GeomAdaptor_Curve aGACurve(GeomLinePtr,
 				     myFirstPar,
 				     myLastPar) ;
-	  myResult = new GeomAdaptor_HCurve(aGACurve);
+	  myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
 	}
 	else {
@@ -605,7 +639,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
 	  NewCurvePtr->SetKnots(BsplineKnots) ;
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
 	  GeomAdaptor_Curve aGACurve(NewCurvePtr);
-	  myResult = new GeomAdaptor_HCurve(aGACurve);
+	  myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
 	}
       }
@@ -643,12 +677,14 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
 
       Standard_Real Tol2 = myTolerance*myTolerance;
       if (VDx.SquareMagnitude() < Tol2 ||
-	  VDy.SquareMagnitude() < Tol2    ) {
-	myIsApprox = Standard_True;
+          VDy.SquareMagnitude() < Tol2 ||
+          VDx.CrossSquareMagnitude(VDy) < Tol2)
+      {
+        myIsApprox = Standard_True;
       }
 
-      if (!myIsApprox && 
-	  gp_Dir(VDx).IsNormal(gp_Dir(VDy),Precision::Angular())) {
+      if (!myIsApprox)
+      {
 	Dx = gp_Dir(VDx);
 	Dy = gp_Dir(VDy);
 	gp_Pnt O    = Axis.Location();
@@ -657,50 +693,120 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
 	gp_Pnt Py   = ProjectPnt(myPlane,myDirection,O.Translated(R2*gp_Vec(Y)));
 	Standard_Real Major = P.Distance(Px);
 	Standard_Real Minor = P.Distance(Py);
-	gp_Ax2 Axe( P, Dx^Dy,Dx);
 
-	if ( Abs( Major - Minor) < Precision::Confusion()) {
-	  myType = GeomAbs_Circle;
- 	  gp_Circ Circ(Axe, Major);
-	  GeomCirclePtr  = new Geom_Circle(Circ);
+        if (myKeepParam)
+        {
+          myIsApprox = !gp_Dir(VDx).IsNormal(gp_Dir(VDy), Precision::Angular());
+        }
+        else
+        {
+          // Since it is not necessary to keep the same parameter for the point on the original and on the projected curves,
+          // we will use the following approach to find axes of the projected ellipse and provide the canonical curve:
+          // https://www.geometrictools.com/Documentation/ParallelProjectionEllipse.pdf
+          math_Matrix aMatrA(1, 2, 1, 2);
+          // A = Jp^T * Pr(Je), where
+          //   Pr(Je) - projection of axes of original ellipse to the target plane
+          //   Jp - X and Y axes of the target plane
+          aMatrA(1, 1) = myPlane.XDirection().XYZ().Dot(VDx.XYZ());
+          aMatrA(1, 2) = myPlane.XDirection().XYZ().Dot(VDy.XYZ());
+          aMatrA(2, 1) = myPlane.YDirection().XYZ().Dot(VDx.XYZ());
+          aMatrA(2, 2) = myPlane.YDirection().XYZ().Dot(VDy.XYZ());
+
+          math_Matrix aMatrDelta2(1, 2, 1, 2, 0.0);
+          //           | 1/MajorRad^2       0       |
+          // Delta^2 = |                            |
+          //           |      0        1/MajorRad^2 |
+          aMatrDelta2(1, 1) = 1.0 / (R1 * R1);
+          aMatrDelta2(2, 2) = 1.0 / (R2 * R2);
+
+          math_Matrix aMatrAInv = aMatrA.Inverse();
+          math_Matrix aMatrM = aMatrAInv.Transposed() * aMatrDelta2 * aMatrAInv;
+
+          // perform eigenvalues calculation
+          math_Jacobi anEigenCalc(aMatrM);
+          if (anEigenCalc.IsDone())
+          {
+            // radii of the projected ellipse
+            Minor = 1.0 / Sqrt(anEigenCalc.Value(1));
+            Major = 1.0 / Sqrt(anEigenCalc.Value(2));
+
+            // calculate the rotation angle for the plane axes to meet the correct axes of the projected ellipse
+            // (swap eigenvectors in respect to major and minor axes)
+            const math_Matrix& anEigenVec = anEigenCalc.Vectors();
+            gp_Trsf2d aTrsfInPlane;
+            aTrsfInPlane.SetValues(anEigenVec(1, 2), anEigenVec(1, 1), 0.0,
+                                   anEigenVec(2, 2), anEigenVec(2, 1), 0.0);
+            gp_Trsf aRot;
+            aRot.SetRotation(gp_Ax1(P, myPlane.Direction()), aTrsfInPlane.RotationPart());
+
+            Dx = myPlane.XDirection().Transformed(aRot);
+            Dy = myPlane.YDirection().Transformed(aRot);
+          }
+          else
+          {
+            myIsApprox = Standard_True;
+          }
+        }
+
+        if (!myIsApprox)
+        {
+          gp_Ax2 Axe(P, Dx^Dy, Dx);
+
+          if (Abs(Major - Minor) < Precision::Confusion()) {
+            myType = GeomAbs_Circle;
+            gp_Circ Circ(Axe, Major);
+            GeomCirclePtr  = new Geom_Circle(Circ);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
-	  GeomAdaptor_Curve aGACurve(GeomCirclePtr);
-	  myResult = new GeomAdaptor_HCurve(aGACurve);
+            GeomAdaptor_Curve aGACurve(GeomCirclePtr);
+            myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
-	}
-	else if ( Major > Minor) {
-	  myType = GeomAbs_Ellipse;
-	  Elips  = gp_Elips( Axe, Major, Minor);
-	  
-          GeomEllipsePtr = new Geom_Ellipse(Elips) ;
+          }
+          else if ( Major > Minor) {
+            myType = GeomAbs_Ellipse;
+            Elips  = gp_Elips( Axe, Major, Minor);
+
+            GeomEllipsePtr = new Geom_Ellipse(Elips);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
-	  GeomAdaptor_Curve aGACurve(GeomEllipsePtr);
-	  myResult = new GeomAdaptor_HCurve(aGACurve);
+            GeomAdaptor_Curve aGACurve(GeomEllipsePtr);
+            myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
-	}
-	else {
-	  myIsApprox = Standard_True;
-	  myType = GeomAbs_BSplineCurve;
-          PerformApprox(myCurve,myPlane,myDirection,ApproxCurve);
-//  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
-	  GeomAdaptor_Curve aGACurve(ApproxCurve);
-	  myResult = new GeomAdaptor_HCurve(aGACurve);
-//  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
-	}
+          }
+          else {
+            myIsApprox = Standard_True;
+          }
+        }
       }
-      else {
-	myIsApprox = Standard_True;
+
+      // No way to build the canonical curve, approximate as B-spline
+      if (myIsApprox)
+      {
 	myType     = GeomAbs_BSplineCurve;
         PerformApprox(myCurve,myPlane,myDirection,ApproxCurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
 	GeomAdaptor_Curve aGACurve(ApproxCurve);
-	myResult = new GeomAdaptor_HCurve(aGACurve);
+	myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
       }
+      else if (GeomCirclePtr || GeomEllipsePtr)
+      {
+        Handle(Geom_Curve) aResultCurve = GeomCirclePtr;
+        if (aResultCurve.IsNull())
+          aResultCurve = GeomEllipsePtr;
+        // start and end parameters of the projected curve
+        Standard_Real aParFirst = myCurve->FirstParameter();
+        Standard_Real aParLast = myCurve->LastParameter();
+        gp_Pnt aPntFirst = ProjectPnt(myPlane, myDirection, myCurve->Value(aParFirst));
+        gp_Pnt aPntLast = ProjectPnt(myPlane, myDirection, myCurve->Value(aParLast));
+        GeomLib_Tool::Parameter(aResultCurve, aPntFirst, Precision::Confusion(), myFirstPar);
+        GeomLib_Tool::Parameter(aResultCurve, aPntLast, Precision::Confusion(), myLastPar);
+        while (myLastPar <= myFirstPar)
+          myLastPar += myResult->Period();
+      }
     }
     break;
   case GeomAbs_Parabola:
     {
+      myKeepParam = Standard_True;
       //     P(u) = O + (u*u)/(4*f) * Xc + u * Yc
       // ==> Q(u) = f(P(u)) 
       //          = f(O) + (u*u)/(4*f) * f(Xc) + u * f(Yc)
@@ -722,7 +828,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
           GeomLinePtr    = new Geom_Line(aLine) ;
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
 	  GeomAdaptor_Curve aGACurve(GeomLinePtr);
-	  myResult = new GeomAdaptor_HCurve(aGACurve);
+	  myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
           alocalIsDone = Standard_True;
 	}
@@ -734,7 +840,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
 	    new Geom_Parabola(aParab) ;
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
 	  GeomAdaptor_Curve aGACurve(GeomParabolaPtr);
-	  myResult = new GeomAdaptor_HCurve(aGACurve);
+	  myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
           alocalIsDone = Standard_True;
 	}
@@ -745,13 +851,14 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
         PerformApprox(myCurve,myPlane,myDirection,ApproxCurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
 	GeomAdaptor_Curve aGACurve(ApproxCurve);
-	myResult = new GeomAdaptor_HCurve(aGACurve);
+	myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
       }
     }
     break;
   case GeomAbs_Hyperbola:
     {
+      myKeepParam = Standard_True;
       //     P(u) = O + R1 * Cosh(u) * Xc + R2 * Sinh(u) * Yc
       // ==> Q(u) = f(P(u)) 
       //          = f(O) + R1 * Cosh(u) * f(Xc) + R2 * Sinh(u) * f(Yc)
@@ -773,7 +880,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
 	  new Geom_Hyperbola(Hypr) ;
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
 	GeomAdaptor_Curve aGACurve(GeomHyperbolaPtr);
-	myResult = new GeomAdaptor_HCurve(aGACurve);
+	myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
       }
       else if ( Yc.Magnitude() < Precision::Confusion()) {
@@ -784,7 +891,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
 	  new Geom_Hyperbola(Hypr) ;
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
 	GeomAdaptor_Curve aGACurve(GeomHyperbolaPtr);
-	myResult = new GeomAdaptor_HCurve(aGACurve);
+	myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
       }
       else if ( Xc.IsNormal(Yc,Precision::Angular())) {
@@ -795,7 +902,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
 	  new Geom_Hyperbola(Hypr) ;
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
 	GeomAdaptor_Curve aGACurve(GeomHyperbolaPtr);
-	myResult = new GeomAdaptor_HCurve(aGACurve);
+	myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
       }
       else {
@@ -804,7 +911,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
         PerformApprox(myCurve,myPlane,myDirection,ApproxCurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
 	GeomAdaptor_Curve aGACurve(ApproxCurve);
-	myResult = new GeomAdaptor_HCurve(aGACurve);
+	myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
       }
     }
@@ -819,6 +926,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
       Handle(Geom_BezierCurve) ProjCu = 
 	Handle(Geom_BezierCurve)::DownCast(BezierCurvePtr->Copy());
 
+      myKeepParam = Standard_True;
       myIsApprox = Standard_False;
       myType = Type;
       for ( Standard_Integer i = 1; i <= NbPoles; i++) {
@@ -828,7 +936,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
       
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
       GeomAdaptor_Curve aGACurve(ProjCu);
-      myResult = new GeomAdaptor_HCurve(aGACurve);
+      myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
     }
     break ;
@@ -842,6 +950,7 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
       Handle(Geom_BSplineCurve) ProjectedBSplinePtr =
 	Handle(Geom_BSplineCurve)::DownCast(BSplineCurvePtr->Copy()) ;
       
+      myKeepParam = Standard_True;
       myIsApprox = Standard_False;
       myType = Type;
       for ( Standard_Integer i = 1; i <= BSplineCurvePtr->NbPoles(); i++) {
@@ -851,18 +960,19 @@ void ProjLib_ProjectOnPlane::Load(const Handle(Adaptor3d_HCurve)&    C,
       
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
       GeomAdaptor_Curve aGACurve(ProjectedBSplinePtr);
-      myResult = new GeomAdaptor_HCurve(aGACurve);
+      myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
     }
     break;
   default:
     {
+      myKeepParam = Standard_True;
       myIsApprox = Standard_True;
       myType     = GeomAbs_BSplineCurve;
       PerformApprox(myCurve,myPlane,myDirection,ApproxCurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:29 2002 Begin
       GeomAdaptor_Curve aGACurve(ApproxCurve);
-      myResult = new GeomAdaptor_HCurve(aGACurve);
+      myResult = new GeomAdaptor_Curve(aGACurve);
 //  Modified by Sergey KHROMOV - Tue Jan 29 16:57:30 2002 End
     }
     break;
@@ -894,7 +1004,7 @@ const gp_Dir& ProjLib_ProjectOnPlane::GetDirection() const
 //purpose  : 
 //=======================================================================
 
-const Handle(Adaptor3d_HCurve)& ProjLib_ProjectOnPlane::GetCurve() const
+const Handle(Adaptor3d_Curve)& ProjLib_ProjectOnPlane::GetCurve() const
 {
   return myCurve;
 }
@@ -904,7 +1014,7 @@ const Handle(Adaptor3d_HCurve)& ProjLib_ProjectOnPlane::GetCurve() const
 //purpose  : 
 //=======================================================================
 
-const Handle(GeomAdaptor_HCurve)& ProjLib_ProjectOnPlane::GetResult() const
+const Handle(GeomAdaptor_Curve)& ProjLib_ProjectOnPlane::GetResult() const
 {
   return myResult;
 }
@@ -976,7 +1086,7 @@ void ProjLib_ProjectOnPlane::Intervals(TColStd_Array1OfReal& T,
 //purpose  : 
 //=======================================================================
 
-Handle(Adaptor3d_HCurve)  
+Handle(Adaptor3d_Curve)  
 ProjLib_ProjectOnPlane::Trim(const Standard_Real First,
 			     const Standard_Real Last,
 			     const Standard_Real Tolerance) const 
